@@ -1,5 +1,3 @@
-# $Id: AuthMgr.pm,v 1.7 2008/10/02 20:46:17 turnstep Exp $
-
 package Net::SSH::Perl::AuthMgr;
 use strict;
 use warnings;
@@ -11,6 +9,7 @@ use Net::SSH::Perl::Auth;
 use Net::SSH::Perl::Constants qw(
     SSH2_MSG_SERVICE_REQUEST
     SSH2_MSG_SERVICE_ACCEPT
+    SSH2_MSG_EXT_INFO
     SSH2_MSG_USERAUTH_BANNER
     SSH2_MSG_USERAUTH_REQUEST
     SSH2_MSG_USERAUTH_SUCCESS
@@ -43,11 +42,26 @@ sub init {
     $packet->send;
 
     $packet = Net::SSH::Perl::Packet->read($ssh);
+    if ($packet->type == SSH2_MSG_EXT_INFO) {
+        $ssh->debug("SSH2_MSG_EXT_INFO received");
+        my $num_ext = $packet->get_int32;
+        while ($num_ext) {
+            my $name = $packet->get_str;
+            my $value = $packet->get_str;
+            $ssh->debug("SSH Extension activated: $name=$value");
+            $ssh->{$name} = $value;
+            $num_ext--;
+        }
+        $packet = Net::SSH::Perl::Packet->read($ssh);
+    }
     croak "Server denied SSH2_MSG_SERVICE_ACCEPT: ", $packet->type
         unless $packet->type == SSH2_MSG_SERVICE_ACCEPT;
     $ssh->debug("Service accepted: " . $packet->get_str . ".");
 
-    $amgr->{agent} = Net::SSH::Perl::Agent->new(2);
+    if (!$ssh->config->get('identities_only')) {
+    	$amgr->{agent} = Net::SSH::Perl::Agent->new(2);
+    }
+
     $amgr->{service} = "ssh-connection";
 
     $amgr->send_auth_none;
