@@ -1,10 +1,8 @@
-# $Id: Config.pm,v 1.22 2008/10/02 20:46:17 turnstep Exp $
-
 package Net::SSH::Perl::Config;
 use strict;
 use warnings;
 
-use Net::SSH::Perl::Constants qw( :protocol );
+use Net::SSH::Perl::Constants qw( :protocol :kex );
 use vars qw( %DIRECTIVES $AUTOLOAD );
 use Carp qw( croak );
 
@@ -13,18 +11,21 @@ use Carp qw( croak );
     Host                    => [ \&_host ],
     BatchMode               => [ \&_batch_mode ],
     ChallengeResponseAuthentication => [ \&_set_yesno, 'auth_ch_res' ],
+    CheckHostIP             => [ \&_set_yesno, 'check_host_ip' ],
     Cipher                  => [ \&_cipher ],
-    Ciphers                 => [ \&_set_str, 'ciphers' ],
+    Ciphers                 => [ \&_set_str, 'ciphers', KEX_DEFAULT_ENCRYPT, KEX_ALL_ENCRYPT ],
     Compression             => [ \&_set_yesno, 'compression' ],
     CompressionLevel        => [ \&_set_str, 'compression_level' ],
     DSAAuthentication       => [ \&_set_yesno, 'auth_dsa' ],
+    FingerprintHash         => [ \&_set_str, 'fingerprint_hash' ],
     GlobalKnownHostsFile    => [ \&_set_str, 'global_known_hosts' ],
     HashKnownHosts          => [ \&_set_yesno, 'hash_known_hosts' ],
-    HostKeyAlgorithms       => [ \&_set_str, 'host_key_algorithms' ],
+    HostKeyAlgorithms       => [ \&_set_str, 'host_key_algorithms', KEX_DEFAULT_PK_ALG, KEX_ALL_PK_ALG ],
     HostName                => [ \&_set_str, 'hostname' ],
     IdentityFile            => [ \&_identity_file ],
-    KexAlgorithms           => [ \&_set_str, 'kex_algorithms' ],
-    MACs                    => [ \&_set_str, 'macs' ],
+    IdentitiesOnly          => [ \&_set_yesno, 'identities_only' ],
+    KexAlgorithms           => [ \&_set_str, 'kex_algorithms', KEX_DEFAULT_KEX, KEX_ALL_KEX ],
+    MACs                    => [ \&_set_str, 'macs', KEX_DEFAULT_MAC, KEX_ALL_MAC ],
     NumberOfPasswordPrompts => [ \&_set_str, 'number_of_password_prompts' ],
     PasswordAuthentication  => [ \&_set_yesno, 'auth_password' ],
     PasswordPromptHost      => [ \&_set_yesno, 'password_prompt_host' ],
@@ -35,6 +36,7 @@ use Carp qw( croak );
     RhostsRSAAuthentication => [ \&_set_yesno, 'auth_rhosts_rsa' ],
     RSAAuthentication       => [ \&_set_yesno, 'auth_rsa' ],
     StrictHostKeyChecking   => [ \&_set_str, 'strict_host_key_checking' ],
+    UpdateHostKeys          => [ \&_set_str, 'update_host_keys' ],
     UsePrivilegedPort       => [ \&_set_yesno, 'privileged' ],
     User                    => [ \&_set_str, 'user' ],
     UserKnownHostsFile      => [ \&_set_str, 'user_known_hosts' ],
@@ -125,6 +127,25 @@ sub _protocol {
 sub _set_str {
     my($cfg, $key, $value) = @_;
     return if exists $cfg->{o}{ $DIRECTIVES{$key}[1] };
+    if ($value =~ s/^-//) {
+        my @list;
+        $value =~ s/\*/\.\*/g;
+        my $defaults = $DIRECTIVES{$key}[2] or return;
+        foreach (split(',', $defaults)) {
+            next if /^$value$/;
+            push @list, $_;
+        }
+        $value = join(',',@list);
+    }
+    elsif ($value =~ s/^\+//) {
+        my @list;
+        $value =~ s/\*/\.\*/g;
+        my $all = $DIRECTIVES{$key}[3] or return;
+        foreach (split(',', $all)) {
+            push @list, $_ if /^$value$/;
+        }
+        $value = join(',', $DIRECTIVES{$key}[2], @list);
+    }
     $cfg->{o}{ $DIRECTIVES{$key}[1] } = $value;
 }
 
