@@ -1,17 +1,13 @@
 package Net::SSH::Perl::Key::Ed25519;
 use strict;
 
-require XSLoader;
-XSLoader::load('Net::SSH::Perl');
-
 use Net::SSH::Perl::Buffer;
-use Net::SSH::Perl::Constants qw( SSH_COMPAT_BUG_SIGBLOB );
 use Crypt::Digest::SHA512 qw( sha512 );
 
 use base qw( Net::SSH::Perl::Key );
 
 use Crypt::PRNG qw( random_bytes );
-use MIME::Base64;
+use Crypt::Misc qw( decode_b64 encode_b64 );
 use Carp qw( croak );
 
 use constant MARK_BEGIN => "-----BEGIN OPENSSH PRIVATE KEY-----\n";
@@ -24,6 +20,11 @@ use constant DEFAULT_ROUNDS => 16;
 use constant DEFAULT_CIPHERNAME => 'aes256-cbc';
 use constant KDFNAME => 'bcrypt';
 
+unless (grep /^Net::SSH::Perl$/, @DynaLoader::dl_modules) {
+        use XSLoader;
+        XSLoader::load('Net::SSH::Perl');
+}
+
 sub ssh_name { 'ssh-ed25519' }
 
 sub init {
@@ -43,7 +44,6 @@ sub init {
 sub keygen {
     my $class = shift;
     my $key = __PACKAGE__->new(undef);
-    $key->{comment} = shift;
     my $secret = random_bytes(ED25519_PK_SZ);
     ($key->{pub},$key->{priv}) = ed25519_generate_keypair($secret);
     $key;
@@ -57,8 +57,9 @@ sub read_private {
     open FH, $key_file or return;
     my $content = do { local $/; <FH> };
     close FH;
-    my $blob = decode_base64(substr($content,length(MARK_BEGIN),
-        length($content)-length(MARK_END)-length(MARK_BEGIN)));
+    $content = substr($content,length(MARK_BEGIN),
+        length($content)-length(MARK_END)-length(MARK_BEGIN));
+    my $blob = decode_b64($content);
     my $str = AUTH_MAGIC;
     croak "Invalid key format" unless $blob =~ /^${str}/;
 
@@ -144,7 +145,7 @@ sub read_private {
     }
 
     my $key = __PACKAGE__->new(undef);
-    $key->{comment} = $comment;
+    $key->comment($comment);
     $key->{pub} = $pub_key;
     $key->{priv} = $priv_key;
     $key;
@@ -207,7 +208,7 @@ sub write_private {
     $kb->put_str($key->ssh_name);
     $kb->put_str($key->{pub});
     $kb->put_str($key->{priv});
-    $kb->put_str($key->{comment});
+    $kb->put_str($key->comment);
     if (my $r = length($kb->bytes) % $blocksize) {
          $kb->put_char(chr($_)) foreach (1..$blocksize-$r);
     }
@@ -219,18 +220,11 @@ sub write_private {
     local *FH;
     open FH, ">$key_file" or die "Cannot write key file";
     print FH MARK_BEGIN;
-    print FH encode_base64($b->bytes); 
+    print FH encode_b64($b->bytes),"\n";
     print FH MARK_END;
     close FH;
 }
 
-sub dump_public {
-    my $key = shift;
-    my $pub = $key->ssh_name . ' ' . encode_base64( $key->as_blob, '');
-    $pub .= ' ' . $key->{comment} if defined $key->{comment};
-    $pub
-}
-
 sub sign {
     my $key = shift;
     my $data = shift;
@@ -346,8 +340,8 @@ Net::SSH::Perl::Key::Ed25519 - Ed25519 key object
 I<Net::SSH::Perl::Key::Ed25519> subclasses I<Net::SSH::Perl::Key>
 to implement an OpenSSH key object.  It uses code taken from the
 SUPERCOP ref10 implementation to do the crypto heavy lifting.
-The I<Net::SSH::Perl::Buffer> class is used to create blobs and 
-transforms those it into a key object and to write keys to an 
+The I<Net::SSH::Perl::Buffer> class is used to create blobs and
+transforms those it into a key object and to write keys to an
 openssh-key-v1 file.
 
 =head1 USAGE
@@ -367,7 +361,7 @@ Returns the signature blob.
 
 Given a signature blob I<$signature> and the original signed data
 I<$data>, attempts to verify the signature using the public key
-portion of I<$key>. 
+portion of I<$key>.
 
 I<$signature> should be an SSH-compatible signature blob, as
 returned from I<sign>; I<$data> should be a string of data, as
